#include "Status.h"
#include "Vram.h"
#include "Main.h"
#include "Sprite.h"
#include "Print.h"
#include "Chars.h"

constexpr byte Char_Remain = Char_Fighter;

static ptr<byte> PrintS(ptr<byte> pVram, ptr<byte> p)
{
    byte c;
    while ((c = *p) != 0) {
        pVram = PrintC(pVram, c);
        ++p;
    }
    return pVram;
}


static ptr<byte> PrintRemain(ptr<byte> pVram)
{
    byte c;
    c = Char_Remain;
    repeat (2) {
        repeat (2) {
            pVram = Put(pVram, c);
            ++c;
        }
        pVram += VramRowSize - 2 * VramStep;
    }
    return pVram + (2 * VramStep - VramRowSize * 2);
}


static ptr<byte> EraseRemain(ptr<byte> pVram)
{
    repeat (2) {
        repeat (2) {
            pVram = Put(pVram, Char_Space);
        }
        pVram += VramRowSize - 2 * VramStep;
    }
    return pVram + (2 * VramStep - VramRowSize * 2);
}


void PrintStatus() 
{
    PrintS(Vram + 0, "SC");
    PrintS(Vram + 10, "HI");
    PrintS(Vram + 20, "ST");
    {
        ptr<byte> pVram;
        pVram = Vram + 26 * VramStep;
        if (RemainCount > 1) {
            byte i;
            i = RemainCount - 1;
            if (i > 2) {
                pVram = PrintRemain(pVram);
                pVram = PrintC(pVram, ' ');
                pVram = PrintC(pVram, i + '0');
            }
            else {
                do {
                    pVram = PrintRemain(pVram);
                    --i;
                } while (i != 0);
                EraseRemain(pVram);
            }
        }
        else {
            EraseRemain(pVram);           
        }
    }
    PrintStage();
    PrintScore();
    // PrintTime();
}

void PrintScore()
{
    ptr<byte> pVram;
    
    pVram = PrintNumber5(Vram + 2 * VramStep , Score);
    PrintC(pVram, '0');

    pVram = PrintNumber5(Vram + 12 * VramStep, HiScore);
    PrintC(pVram, '0');
}

void PrintStage()
{
    PrintByteNumber2(Vram + 22 * VramStep, CurrentStage + 1);
}

// void PrintTime()
// {
//     PrintByteNumber3(Vram + VramRowSize + 23, StageTime);
// }

static void EraseWindow() 
{
    constexpr byte width = 12;
    constexpr byte height = 3;    
    ptr<byte> pVram;

    pVram = Vram + VramRowSize * 11 + 10;
    repeat (height) {
        repeat (width) {
            pVram = PrintC(pVram, ' ');
        }
        pVram += VramRowSize - width;
    }
}

void PrintGameOver()
{
    EraseWindow();
    PrintS(Vram + VramRowSize * 12 + 11, "GAME OVER");
}

void PrintTimeUp() 
{
    EraseWindow();
    PrintS(Vram + VramRowSize * 12 + 12, "TIME UP");
}

void Title()
{
    static const byte[] TitleBytes = {
        //	I
        0x04, 0x0d, 0x07, 0x01, 
        0x00, 0x0c, 0x03, 0x00, 
        0x00, 0x0c, 0x03, 0x00, 
        0x04, 0x05, 0x05, 0x01, 
        //	M
        0x0c, 0x07, 0x07, 0x0b, 
        0x0c, 0x03, 0x03, 0x0f, 
        0x0c, 0x03, 0x03, 0x0f, 
        0x04, 0x01, 0x01, 0x05, 
        //	P
        0x0c, 0x07, 0x05, 0x0b, 
        0x0c, 0x03, 0x00, 0x0f, 
        0x0c, 0x07, 0x05, 0x01, 
        0x04, 0x01, 0x00, 0x00, 
        //	E
        0x0c, 0x07, 0x05, 0x01, 
        0x0c, 0x0b, 0x0a, 0x02, 
        0x0c, 0x03, 0x00, 0x00, 
        0x04, 0x05, 0x05, 0x01, 
        //	T
        0x04, 0x0d, 0x07, 0x01, 
        0x00, 0x0c, 0x03, 0x00, 
        0x00, 0x0c, 0x03, 0x00, 
        0x00, 0x04, 0x01, 0x00, 
        //	U
        0x0c, 0x03, 0x00, 0x0f, 
        0x0c, 0x03, 0x00, 0x0f, 
        0x0c, 0x03, 0x00, 0x0f, 
        0x00, 0x05, 0x05, 0x01, 
        //	S
        0x08, 0x07, 0x05, 0x0b, 
        0x04, 0x0b, 0x0a, 0x02, 
        0x08, 0x02, 0x00, 0x0f, 
        0x00, 0x05, 0x05, 0x01, 
    };

    ClearScreen(); 
    HideAllSprites();
    PrintStatus();
    {
        ptr<byte> p;
        ptr<byte> pVram;
        pVram = Vram + VramRowSize * 7 + (32 - 4 * 7) / 2;
        p = TitleBytes;
        repeat (7) {
            repeat (4) {
                repeat (4) {
                    pVram = Put(pVram, *p);
                    ++p;
                }
                pVram += VramRowSize - 4;
            }
            pVram += 4 - VramRowSize * 4;
        }
    }
    PrintS(Vram + VramRowSize * 16 + 9, "PUSH SPACE KEY");
    PrintS(Vram + VramRowSize * 18 + 9, "OR SHIFT KEY");
    // {
    //     byte c;
    //     word v;
    //     v = pVram+32*10;
    //     c = 0;
    //     repeat (0) {
    //         v = Put(v, c);
    //         ++c;
    //     }
    // }
}
